# -*- coding: utf-8 -*-
"""
Subpackage containing the commands for the application.

:author: Joe Joyce <joe@decafjoe.com>
:copyright: Copyright (c) Joe Joyce and contributors, 2016-2018.
:license: BSD
"""
import safe.cmd.copy
import safe.cmd.drop.account
import safe.cmd.drop.policy
import safe.cmd.gen.bare
import safe.cmd.gen.per_policy
import safe.cmd.init
import safe.cmd.ipy
import safe.cmd.list
import safe.cmd.new.account
import safe.cmd.new.policy
import safe.cmd.shell
import safe.cmd.update.account
import safe.cmd.update.policy  # noqa: F401
